@@ -13,6 +13,7 @@ hlist_head``) in :linux:`include/linux/list.h`.
 from typing import Iterator, Union
 
 from drgn import NULL, Object, Type, container_of
+from drgn.helpers import ValidationError
 
 __all__ = (
     "hlist_empty",
@@ -29,6 +30,9 @@ __all__ = (
     "list_last_entry",
     "list_next_entry",
     "list_prev_entry",
+    "validate_list",
+    "validate_list_for_each",
+    "validate_list_for_each_entry",
 )
 
 
@@ -186,6 +190,57 @@ def list_for_each_entry_reverse(
         yield container_of(pos, type, member)
 
 
+def validate_list(head: Object) -> None:
+    """
+    Validate that the ``next`` and ``prev`` pointers in a list are consistent.
+
+    :param head: ``struct list_head *``
+    :raises ValidationError: if the list is invalid
+    """
+    for _ in validate_list_for_each(head):
+        pass
+
+
+def validate_list_for_each(head: Object) -> Iterator[Object]:
+    """
+    Like :func:`list_for_each()`, but validates the list like
+    :func:`validate_list()` while iterating.
+
+    :param head: ``struct list_head *``
+    :raises ValidationError: if the list is invalid
+    """
+    head = head.read_()
+    pos = head.next.read_()
+    while pos != head:
+        yield pos
+        next = pos.next.read_()
+        next_prev = next.prev.read_()
+        if next_prev != pos:
+            raise ValidationError(
+                f"{pos.format_(dereference=False, symbolize=False)}"
+                f" next {next.format_(dereference=False, symbolize=False, type_name=False)}"
+                f" has prev {next_prev.format_(dereference=False, symbolize=False, type_name=False)}"
+            )
+        pos = next
+
+
+def validate_list_for_each_entry(
+    type: Union[str, Type], head: Object, member: str
+) -> Iterator[Object]:
+    """
+    Like :func:`list_for_each_entry()`, but validates the list like
+    :func:`validate_list()` while iterating.
+
+    :param type: Entry type.
+    :param head: ``struct list_head *``
+    :param member: Name of list node member in entry type.
+    :raises ValidationError: if the list is invalid
+    """
+    type = head.prog_.type(type)
+    for pos in validate_list_for_each(head):
+        yield container_of(pos, type, member)
+
+
 def hlist_empty(head: Object) -> bool:
     """
     Return whether a hash list is empty.
