@@ -17,28 +17,7 @@
 #include "type.h"
 #include "util.h"
 
-#define DRGN_ARCH_REGISTER_LAYOUT					\
-	/* The psABI calls this the return address (RA) register. */	\
-	DRGN_REGISTER_LAYOUT(rip, 8, 16)				\
-	DRGN_REGISTER_LAYOUT(rsp, 8, 7)					\
-	/* The remaining layout matches struct pt_regs. */		\
-	DRGN_REGISTER_LAYOUT(r15, 8, 15)				\
-	DRGN_REGISTER_LAYOUT(r14, 8, 14)				\
-	DRGN_REGISTER_LAYOUT(r13, 8, 13)				\
-	DRGN_REGISTER_LAYOUT(r12, 8, 12)				\
-	DRGN_REGISTER_LAYOUT(rbp, 8, 6)					\
-	DRGN_REGISTER_LAYOUT(rbx, 8, 3)					\
-	DRGN_REGISTER_LAYOUT(r11, 8, 11)				\
-	DRGN_REGISTER_LAYOUT(r10, 8, 10)				\
-	DRGN_REGISTER_LAYOUT(r9, 8, 9)					\
-	DRGN_REGISTER_LAYOUT(r8, 8, 8)					\
-	DRGN_REGISTER_LAYOUT(rax, 8, 0)					\
-	DRGN_REGISTER_LAYOUT(rcx, 8, 2)					\
-	DRGN_REGISTER_LAYOUT(rdx, 8, 1)					\
-	DRGN_REGISTER_LAYOUT(rsi, 8, 4)					\
-	DRGN_REGISTER_LAYOUT(rdi, 8, 5)
-
-#include "arch_x86_64.inc"
+#include "arch_x86_64_defs.inc"
 
 static const struct drgn_cfi_row default_dwarf_cfi_row_x86_64 = DRGN_CFI_ROW(
 	/*
@@ -175,6 +154,9 @@ orc_to_cfi_x86_64(const struct drgn_orc_entry *orc,
 		SET_AT_CFA_RULE(rdx, 96);
 		SET_AT_CFA_RULE(rsi, 104);
 		SET_AT_CFA_RULE(rdi, 112);
+		SET_AT_CFA_RULE(cs, 136);
+		SET_AT_CFA_RULE(rflags, 144);
+		SET_AT_CFA_RULE(ss, 160);
 		*interrupted_ret = true;
 		break;
 	case DRGN_ORC_TYPE_REGS_PARTIAL:
@@ -266,8 +248,8 @@ get_registers_from_frame_pointer(struct drgn_program *prog,
 	if (!regs)
 		return &drgn_enomem;
 	drgn_register_state_set_from_buffer(regs, rip, &frame[1]);
-	drgn_register_state_set_from_integer(prog, regs, rsp,
-					     frame_pointer + sizeof(frame));
+	drgn_register_state_set_from_u64(prog, regs, rsp,
+					 frame_pointer + sizeof(frame));
 	drgn_register_state_set_from_buffer(regs, rbp, &frame[0]);
 	drgn_register_state_set_pc_from_register(prog, regs, rip);
 	*ret = regs;
@@ -281,15 +263,12 @@ fallback_unwind_x86_64(struct drgn_program *prog,
 {
 	struct drgn_error *err;
 
-	if (!drgn_register_state_has_register(regs, DRGN_REGISTER_NUMBER(rbp)))
+	struct optional_uint64 rbp =
+		drgn_register_state_get_u64(prog, regs, rbp);
+	if (!rbp.has_value)
 		return &drgn_stop;
-	bool little_endian = drgn_platform_is_little_endian(&prog->platform);
-	uint64_t rbp;
-	copy_lsbytes(&rbp, sizeof(rbp), HOST_LITTLE_ENDIAN,
-		     &regs->buf[DRGN_REGISTER_OFFSET(rbp)],
-		     DRGN_REGISTER_SIZE(rbp), little_endian);
 
-	err = get_registers_from_frame_pointer(prog, rbp, ret);
+	err = get_registers_from_frame_pointer(prog, rbp.value, ret);
 	if (err) {
 		if (err->code == DRGN_ERROR_FAULT) {
 			drgn_error_destroy(err);
@@ -297,32 +276,45 @@ fallback_unwind_x86_64(struct drgn_program *prog,
 		}
 		return err;
 	}
-	drgn_register_state_set_cfa(prog, regs, rbp + 16);
+	drgn_register_state_set_cfa(prog, regs, rbp.value + 16);
 	return NULL;
 }
 
-/*
- * The in-kernel struct pt_regs, UAPI struct pt_regs, elf_gregset_t, and struct
- * user_regs_struct all have the same layout.
- */
+// elf_gregset_t (in NT_PRSTATUS) and struct user_regs_struct have the same
+// layout. struct pt_regs is a prefix of that layout which elides several
+// segment registers. full_regset tells us which one we were given; true is
+// elf_gregset_t, false is struct pt_regs.
 static struct drgn_error *
 get_initial_registers_from_struct_x86_64(struct drgn_program *prog,
 					 const void *buf, size_t size,
+					 bool full_regset,
 					 struct drgn_register_state **ret)
 {
-	if (size < 160) {
+	if (size < (full_regset ? 216 : 168)) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "registers are truncated");
 	}
 
-	struct drgn_register_state *regs =
-		drgn_register_state_create(rdi, true);
+	struct drgn_register_state *regs;
+	if (full_regset)
+		regs = drgn_register_state_create(gs, true);
+	else
+		regs = drgn_register_state_create(ss, true);
 	if (!regs)
 		return &drgn_enomem;
 
 	drgn_register_state_set_from_buffer(regs, rip, (uint64_t *)buf + 16);
 	drgn_register_state_set_from_buffer(regs, rsp, (uint64_t *)buf + 19);
 	drgn_register_state_set_range_from_buffer(regs, r15, rdi, buf);
+	drgn_register_state_set_range_from_buffer(regs, cs, rflags,
+						  (uint64_t *)buf + 17);
+	if (full_regset) {
+		drgn_register_state_set_range_from_buffer(regs, ss, gs,
+							  (uint64_t *)buf + 20);
+	} else {
+		drgn_register_state_set_from_buffer(regs, ss,
+						    (uint64_t *)buf + 20);
+	}
 	drgn_register_state_set_pc_from_register(prog, regs, rip);
 
 	*ret = regs;
@@ -336,7 +328,7 @@ pt_regs_get_initial_registers_x86_64(const struct drgn_object *obj,
 	return get_initial_registers_from_struct_x86_64(drgn_object_program(obj),
 							drgn_object_buffer(obj),
 							drgn_object_size(obj),
-							ret);
+							false, ret);
 }
 
 static struct drgn_error *
@@ -344,13 +336,16 @@ prstatus_get_initial_registers_x86_64(struct drgn_program *prog,
 				      const void *prstatus, size_t size,
 				      struct drgn_register_state **ret)
 {
-	if (size < 112) {
+	// offsetof(struct elf_prstatus, pr_reg)
+	static const size_t pr_reg_offset = 112;
+	if (size < pr_reg_offset) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "NT_PRSTATUS is truncated");
 	}
 	return get_initial_registers_from_struct_x86_64(prog,
-							(char *)prstatus + 112,
-							size - 112, ret);
+							(char *)prstatus + pr_reg_offset,
+							size - pr_reg_offset,
+							true, ret);
 }
 
 static struct drgn_error *
@@ -398,8 +393,7 @@ get_initial_registers_inactive_task_frame(struct drgn_object *frame_obj,
 
 #undef COPY_REGISTER
 
-	drgn_register_state_set_from_integer(prog, regs,
-					     rsp, address + frame_size);
+	drgn_register_state_set_from_u64(prog, regs, rsp, address + frame_size);
 	drgn_register_state_set_pc_from_register(prog, regs, rip);
 
 	*ret = regs;
@@ -506,108 +500,6 @@ apply_elf_reloc_x86_64(const struct drgn_relocating_section *relocating,
 	}
 }
 
-static struct drgn_error *
-linux_kernel_get_page_offset_x86_64(struct drgn_object *ret)
-{
-	struct drgn_error *err;
-	struct drgn_program *prog = drgn_object_program(ret);
-
-	struct drgn_qualified_type qualified_type;
-	err = drgn_program_find_primitive_type(prog, DRGN_C_TYPE_UNSIGNED_LONG,
-					       &qualified_type.type);
-	if (err)
-		return err;
-	qualified_type.qualifiers = 0;
-
-	/*
-	 * If KASLR is enabled, PAGE_OFFSET is easily available via
-	 * page_offset_base.
-	 */
-	struct drgn_object tmp;
-	drgn_object_init(&tmp, prog);
-	err = drgn_program_find_object(prog, "page_offset_base", NULL,
-				       DRGN_FIND_OBJECT_VARIABLE, &tmp);
-	if (!err) {
-		err = drgn_object_cast(ret, qualified_type, &tmp);
-		goto out;
-	}
-	if (err->code == DRGN_ERROR_LOOKUP)
-		drgn_error_destroy(err);
-	else
-		goto out;
-
-	/*
-	 * If not, we determine it based on the kernel page table. Before Linux
-	 * kernel commit d52888aa2753 ("x86/mm: Move LDT remap out of KASLR
-	 * region on 5-level paging") (in v4.20), PAGE_OFFSET was pgd slot 272.
-	 * After that, it is pgd slot 273, and slot 272 is empty (reserved for
-	 * Local Descriptor Table mappings for userspace tasks).
-	 */
-	uint64_t pgd;
-	err = drgn_program_read_u64(prog,
-				    prog->vmcoreinfo.swapper_pg_dir + 272 * 8,
-				    false, &pgd);
-	if (err)
-		goto out;
-	uint64_t value;
-	if (pgd) {
-		if (prog->vmcoreinfo.pgtable_l5_enabled)
-			value = UINT64_C(0xff10000000000000);
-		else
-			value = UINT64_C(0xffff880000000000);
-	} else {
-		if (prog->vmcoreinfo.pgtable_l5_enabled)
-			value = UINT64_C(0xff11000000000000);
-		else
-			value = UINT64_C(0xffff888000000000);
-	}
-	err = drgn_object_set_unsigned(ret, qualified_type, value, 0);
-
-out:
-	drgn_object_deinit(&tmp);
-	return err;
-}
-
-static struct drgn_error *
-linux_kernel_get_vmemmap_x86_64(struct drgn_object *ret)
-{
-
-	struct drgn_error *err;
-	struct drgn_program *prog = drgn_object_program(ret);
-
-	struct drgn_qualified_type qualified_type;
-	err = drgn_program_find_type(prog, "struct page *", NULL,
-				     &qualified_type);
-	if (err)
-		return err;
-
-	/* If KASLR is enabled, vmemmap is vmemmap_base. */
-	struct drgn_object tmp;
-	drgn_object_init(&tmp, prog);
-	err = drgn_program_find_object(prog, "vmemmap_base", NULL,
-				       DRGN_FIND_OBJECT_VARIABLE, &tmp);
-	if (!err) {
-		err = drgn_object_cast(ret, qualified_type, &tmp);
-		goto out;
-	}
-	if (err->code == DRGN_ERROR_LOOKUP)
-		drgn_error_destroy(err);
-	else
-		goto out;
-
-	/* Otherwise, it depends on whether we have 5-level page tables. */
-	uint64_t value;
-	if (prog->vmcoreinfo.pgtable_l5_enabled)
-		value = UINT64_C(0xffd4000000000000);
-	else
-		value = UINT64_C(0xffffea0000000000);
-	err = drgn_object_set_unsigned(ret, qualified_type, value, 0);
-
-out:
-	drgn_object_deinit(&tmp);
-	return err;
-}
-
 static struct drgn_error *
 linux_kernel_live_direct_mapping_fallback_x86_64(struct drgn_program *prog,
 						 uint64_t *address_ret,
@@ -635,19 +527,40 @@ linux_kernel_live_direct_mapping_fallback_x86_64(struct drgn_program *prog,
 }
 
 struct pgtable_iterator_x86_64 {
+	struct pgtable_iterator it;
 	uint16_t index[5];
 	uint64_t table[5][512];
 };
 
-static void pgtable_iterator_arch_init_x86_64(void *buf)
+static struct drgn_error *
+linux_kernel_pgtable_iterator_create_x86_64(struct drgn_program *prog,
+					    struct pgtable_iterator **ret)
 {
-	struct pgtable_iterator_x86_64 *arch = buf;
-	memset(arch->index, 0xff, sizeof(arch->index));
-	memset(arch->table, 0, sizeof(arch->table));
+	struct pgtable_iterator_x86_64 *it = malloc(sizeof(*it));
+	if (!it)
+		return &drgn_enomem;
+	*ret = &it->it;
+	return NULL;
+}
+
+
+static void linux_kernel_pgtable_iterator_destroy_x86_64(struct pgtable_iterator *_it)
+{
+	free(container_of(_it, struct pgtable_iterator_x86_64, it));
+}
+
+static void
+linux_kernel_pgtable_iterator_init_x86_64(struct drgn_program *prog,
+					  struct pgtable_iterator *_it)
+{
+	struct pgtable_iterator_x86_64 *it =
+		container_of(_it, struct pgtable_iterator_x86_64, it);
+	memset(it->index, 0xff, sizeof(it->index));
 }
 
 static struct drgn_error *
-linux_kernel_pgtable_iterator_next_x86_64(struct pgtable_iterator *it,
+linux_kernel_pgtable_iterator_next_x86_64(struct drgn_program *prog,
+					  struct pgtable_iterator *_it,
 					  uint64_t *virt_addr_ret,
 					  uint64_t *phys_addr_ret)
 {
@@ -658,14 +571,28 @@ linux_kernel_pgtable_iterator_next_x86_64(struct pgtable_iterator *it,
 	static const uint64_t PSE = 0x80; /* a.k.a. huge page */
 	static const uint64_t ADDRESS_MASK = UINT64_C(0xffffffffff000);
 	struct drgn_error *err;
-	struct drgn_program *prog = it->prog;
-	struct pgtable_iterator_x86_64 *arch = (void *)it->arch;
-	int levels = prog->vmcoreinfo.pgtable_l5_enabled ? 5 : 4, level;
 	bool bswap = drgn_platform_bswap(&prog->platform);
+	struct pgtable_iterator_x86_64 *it =
+		container_of(_it, struct pgtable_iterator_x86_64, it);
+	uint64_t virt_addr = it->it.virt_addr;
+	int levels = prog->vmcoreinfo.pgtable_l5_enabled ? 5 : 4, level;
+
+	uint64_t start_non_canonical =
+		(UINT64_C(1) <<
+		 (PAGE_SHIFT + PGTABLE_SHIFT * levels - 1));
+	uint64_t end_non_canonical =
+		(UINT64_MAX <<
+		 (PAGE_SHIFT + PGTABLE_SHIFT * levels - 1));
+	if (virt_addr >= start_non_canonical && virt_addr < end_non_canonical) {
+		*virt_addr_ret = start_non_canonical;
+		*phys_addr_ret = UINT64_MAX;
+		it->it.virt_addr = end_non_canonical;
+		return NULL;
+	}
 
 	/* Find the lowest level with cached entries. */
 	for (level = 0; level < levels; level++) {
-		if (arch->index[level] < array_size(arch->table[level]))
+		if (it->index[level] < array_size(it->table[level]))
 			break;
 	}
 	/* For every level below that, refill the cache/return pages. */
@@ -674,24 +601,10 @@ linux_kernel_pgtable_iterator_next_x86_64(struct pgtable_iterator *it,
 		bool table_physical;
 		uint16_t index;
 		if (level == levels) {
-			uint64_t start_non_canonical, end_non_canonical;
-			start_non_canonical = (UINT64_C(1) <<
-					       (PAGE_SHIFT +
-						PGTABLE_SHIFT * levels - 1));
-			end_non_canonical = (UINT64_MAX <<
-					     (PAGE_SHIFT +
-					      PGTABLE_SHIFT * levels - 1));
-			if (it->virt_addr >= start_non_canonical &&
-			    it->virt_addr < end_non_canonical) {
-				*virt_addr_ret = start_non_canonical;
-				*phys_addr_ret = UINT64_MAX;
-				it->virt_addr = end_non_canonical;
-				return NULL;
-			}
-			table = it->pgtable;
+			table = it->it.pgtable;
 			table_physical = false;
 		} else {
-			uint64_t entry = arch->table[level][arch->index[level]++];
+			uint64_t entry = it->table[level][it->index[level]++];
 			if (bswap)
 				entry = bswap_64(entry);
 			table = entry & ADDRESS_MASK;
@@ -699,30 +612,30 @@ linux_kernel_pgtable_iterator_next_x86_64(struct pgtable_iterator *it,
 				uint64_t mask = (UINT64_C(1) <<
 						 (PAGE_SHIFT +
 						  PGTABLE_SHIFT * level)) - 1;
-				*virt_addr_ret = it->virt_addr & ~mask;
+				*virt_addr_ret = virt_addr & ~mask;
 				if (entry & PRESENT)
 					*phys_addr_ret = table & ~mask;
 				else
 					*phys_addr_ret = UINT64_MAX;
-				it->virt_addr = (it->virt_addr | mask) + 1;
+				it->it.virt_addr = (virt_addr | mask) + 1;
 				return NULL;
 			}
 			table_physical = true;
 		}
-		index = (it->virt_addr >>
+		index = (virt_addr >>
 			 (PAGE_SHIFT + PGTABLE_SHIFT * (level - 1))) & PGTABLE_MASK;
 		/*
 		 * It's only marginally more expensive to read 4096 bytes than 8
 		 * bytes, so we always read to the end of the table.
 		 */
 		err = drgn_program_read_memory(prog,
-					       &arch->table[level - 1][index],
+					       &it->table[level - 1][index],
 					       table + 8 * index,
-					       sizeof(arch->table[0]) - 8 * index,
+					       sizeof(it->table[0]) - 8 * index,
 					       table_physical);
 		if (err)
 			return err;
-		arch->index[level - 1] = index;
+		it->index[level - 1] = index;
 	}
 }
 
@@ -740,12 +653,14 @@ const struct drgn_architecture_info arch_info_x86_64 = {
 	.linux_kernel_get_initial_registers =
 		linux_kernel_get_initial_registers_x86_64,
 	.apply_elf_reloc = apply_elf_reloc_x86_64,
-	.linux_kernel_get_page_offset = linux_kernel_get_page_offset_x86_64,
-	.linux_kernel_get_vmemmap = linux_kernel_get_vmemmap_x86_64,
 	.linux_kernel_live_direct_mapping_fallback =
 		linux_kernel_live_direct_mapping_fallback_x86_64,
-	.pgtable_iterator_arch_size = sizeof(struct pgtable_iterator_x86_64),
-	.pgtable_iterator_arch_init = pgtable_iterator_arch_init_x86_64,
+	.linux_kernel_pgtable_iterator_create =
+		linux_kernel_pgtable_iterator_create_x86_64,
+	.linux_kernel_pgtable_iterator_destroy =
+		linux_kernel_pgtable_iterator_destroy_x86_64,
+	.linux_kernel_pgtable_iterator_init =
+		linux_kernel_pgtable_iterator_init_x86_64,
 	.linux_kernel_pgtable_iterator_next =
 		linux_kernel_pgtable_iterator_next_x86_64,
 };
