@@ -7,14 +7,14 @@
 #include "drgn.h"
 
 struct drgn_debug_info_load_state;
-struct vmcoreinfo;
 
 struct drgn_error *read_memory_via_pgtable(void *buf, uint64_t address,
 					   size_t count, uint64_t offset,
 					   void *arg, bool physical);
 
-struct drgn_error *parse_vmcoreinfo(const char *desc, size_t descsz,
-				    struct vmcoreinfo *ret);
+struct drgn_error *drgn_program_parse_vmcoreinfo(struct drgn_program *prog,
+						 const char *desc,
+						 size_t descsz);
 
 struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
 					     unsigned long *ret);
