@@ -12,61 +12,148 @@
 #include "program.h"
 #include "util.h"
 
+static void end_virtual_address_translation(struct drgn_program *prog)
+{
+	prog->in_address_translation = false;
+}
+
+static struct drgn_error *
+begin_virtual_address_translation(struct drgn_program *prog, uint64_t pgtable,
+				  uint64_t virt_addr)
+{
+	struct drgn_error *err;
+
+	if (prog->in_address_translation) {
+		return drgn_error_create_fault("recursive address translation; "
+					       "page table may be missing from core dump",
+					       virt_addr);
+	}
+	prog->in_address_translation = true;
+	if (!prog->pgtable_it) {
+		if (!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)) {
+			err = drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						"virtual address translation is only available for the Linux kernel");
+			goto err;
+		}
+		if (!prog->has_platform) {
+			err = drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						"cannot do virtual address translation without platform");
+			goto err;
+		}
+		if (!prog->platform.arch->linux_kernel_pgtable_iterator_next) {
+			err = drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
+						"virtual address translation is not implemented for %s architecture",
+						prog->platform.arch->name);
+			goto err;
+		}
+		err = prog->platform.arch->linux_kernel_pgtable_iterator_create(prog,
+										&prog->pgtable_it);
+		if (err) {
+			prog->pgtable_it = NULL;
+			goto err;
+		}
+	}
+	prog->pgtable_it->pgtable = pgtable;
+	prog->pgtable_it->virt_addr = virt_addr;
+	prog->platform.arch->linux_kernel_pgtable_iterator_init(prog, prog->pgtable_it);
+	return NULL;
+
+err:
+	end_virtual_address_translation(prog);
+	return err;
+}
+
+struct drgn_error *linux_helper_direct_mapping_offset(struct drgn_program *prog,
+						      uint64_t *ret)
+{
+	struct drgn_error *err;
+
+	if (prog->direct_mapping_offset_cached) {
+		*ret = prog->direct_mapping_offset;
+		return NULL;
+	}
+
+	// The direct mapping offset can vary depending on architecture, kernel
+	// version, configuration options, and KASLR. Rather than dealing with
+	// all of that, get a virtual address in the direct mapping and
+	// translate it to a physical address via the page table. The difference
+	// is the offset.
+	//
+	// The virtual address we pick doesn't matter as long as:
+	//
+	// 1. It is in the direct mapping on all configurations of all supported
+	//    kernel versions on all architectures.
+	// 2. That is unlikely to change in the future.
+	//
+	// This is our current arbitrary choice.
+	static const char direct_mapping_variable[] = "saved_command_line";
+
+	struct drgn_object tmp;
+	drgn_object_init(&tmp, prog);
+	err = drgn_program_find_object(prog, direct_mapping_variable, NULL,
+				       DRGN_FIND_OBJECT_VARIABLE, &tmp);
+	uint64_t virt_addr;
+	if (!err) {
+		err = drgn_object_read_unsigned(&tmp, &virt_addr);
+	} else if (err->code == DRGN_ERROR_LOOKUP) {
+		// Avoid a confusing error message with our arbitrary variable
+		// name.
+		drgn_error_destroy(err);
+		err = drgn_error_create(DRGN_ERROR_OTHER,
+					"could not find variable in direct mapping");
+	}
+	drgn_object_deinit(&tmp);
+	if (err)
+		return err;
+
+	err = begin_virtual_address_translation(prog,
+						prog->vmcoreinfo.swapper_pg_dir,
+						virt_addr);
+	if (err)
+		return err;
+	uint64_t start_virt_addr, start_phys_addr;
+	err = prog->platform.arch->linux_kernel_pgtable_iterator_next(prog,
+								      prog->pgtable_it,
+								      &start_virt_addr,
+								      &start_phys_addr);
+	if (err)
+		goto out;
+	if (start_phys_addr == UINT64_MAX) {
+		err = drgn_error_create(DRGN_ERROR_OTHER,
+					"could not determine direct mapping offset");
+		goto out;
+	}
+	prog->direct_mapping_offset = start_virt_addr - start_phys_addr;
+	prog->direct_mapping_offset_cached = true;
+	*ret = prog->direct_mapping_offset;
+	err = NULL;
+out:
+	end_virtual_address_translation(prog);
+	return err;
+}
+
 struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 					uint64_t pgtable, uint64_t virt_addr,
 					void *buf, size_t count)
 {
 	struct drgn_error *err;
-	struct pgtable_iterator *it;
-	pgtable_iterator_next_fn *next;
+
+	err = begin_virtual_address_translation(prog, pgtable, virt_addr);
+	if (err)
+		return err;
+	if (!count) {
+		err = NULL;
+		goto out;
+	}
+
+	struct pgtable_iterator *it = prog->pgtable_it;
+	pgtable_iterator_next_fn *next =
+		prog->platform.arch->linux_kernel_pgtable_iterator_next;
 	uint64_t read_addr = 0;
 	size_t read_size = 0;
-
-	if (!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "virtual address translation is only available for the Linux kernel");
-	}
-	if (!prog->has_platform) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "cannot do virtual address translation without platform");
-	}
-	if (!prog->platform.arch->linux_kernel_pgtable_iterator_next) {
-		return drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
-					 "virtual address translation is not implemented for %s architecture",
-					 prog->platform.arch->name);
-	}
-
-	if (!count)
-		return NULL;
-
-	if (prog->pgtable_it_in_use) {
-		return drgn_error_create_fault("recursive address translation; "
-					       "page table may be missing from core dump",
-					       virt_addr);
-	}
-
-	if (prog->pgtable_it) {
-		it = prog->pgtable_it;
-	} else {
-		it = malloc(sizeof(*it) +
-			    prog->platform.arch->pgtable_iterator_arch_size);
-		if (!it)
-			return &drgn_enomem;
-		prog->pgtable_it = it;
-		it->prog = prog;
-	}
-	it->pgtable = pgtable;
-	it->virt_addr = virt_addr;
-	prog->pgtable_it_in_use = true;
-	prog->platform.arch->pgtable_iterator_arch_init(it->arch);
-	next = prog->platform.arch->linux_kernel_pgtable_iterator_next;
 	do {
-		uint64_t virt_addr, start_virt_addr, end_virt_addr;
-		uint64_t start_phys_addr, end_phys_addr;
-		size_t n;
-
-		virt_addr = it->virt_addr;
-		err = next(it, &start_virt_addr, &start_phys_addr);
+		uint64_t start_virt_addr, start_phys_addr;
+		err = next(prog, it, &start_virt_addr, &start_phys_addr);
 		if (err)
 			break;
 		if (start_phys_addr == UINT64_MAX) {
@@ -74,10 +161,11 @@ struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 						      virt_addr);
 			break;
 		}
-		end_virt_addr = it->virt_addr;
-		end_phys_addr = start_phys_addr + (end_virt_addr - start_virt_addr);
-		n = min(end_virt_addr - virt_addr, (uint64_t)count);
-		if (read_size && end_phys_addr == read_addr + read_size) {
+
+		uint64_t phys_addr =
+			start_phys_addr + (virt_addr - start_virt_addr);
+		size_t n = min(it->virt_addr - virt_addr, (uint64_t)count);
+		if (read_size && phys_addr == read_addr + read_size) {
 			read_size += n;
 		} else {
 			if (read_size) {
@@ -88,16 +176,18 @@ struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 					break;
 				buf = (char *)buf + read_size;
 			}
-			read_addr = start_phys_addr + (virt_addr - start_virt_addr);
+			read_addr = phys_addr;
 			read_size = n;
 		}
+		virt_addr = it->virt_addr;
 		count -= n;
 	} while (count);
 	if (!err) {
 		err = drgn_program_read_memory(prog, buf, read_addr, read_size,
 					       true);
 	}
-	prog->pgtable_it_in_use = false;
+out:
+	end_virtual_address_translation(prog);
 	return err;
 }
 
