@@ -18,7 +18,6 @@ struct drgn_register {
 	const char * const *names;
 	size_t num_names;
 	drgn_register_number regno;
-	uint64_t dwarf_number;
 };
 
 struct drgn_register_layout {
@@ -26,6 +25,10 @@ struct drgn_register_layout {
 	uint32_t size;
 };
 
+// This is an ugly layering violation needed for DW_CFA_AARCH64_negate_ra_state.
+// We enforce that it stays up to date with a static_assert() in arch_aarch64.c.
+#define DRGN_AARCH64_RA_SIGN_STATE_REGNO 0
+
 /* ELF section to apply relocations to. */
 struct drgn_relocating_section {
 	char *buf;
@@ -77,13 +80,10 @@ apply_elf_reloc_fn(const struct drgn_relocating_section *relocating,
 
 /* Page table iterator. */
 struct pgtable_iterator {
-	struct drgn_program *prog;
 	/* Address of the top-level page table to iterate. */
 	uint64_t pgtable;
 	/* Current virtual address to translate. */
 	uint64_t virt_addr;
-	/* Architecture-specific data. */
-	char arch[];
 };
 
 /*
@@ -110,17 +110,29 @@ struct pgtable_iterator {
  * maps to, or @c UINT64_MAX if it is not mapped.
  */
 typedef struct drgn_error *
-(pgtable_iterator_next_fn)(struct pgtable_iterator *it, uint64_t *virt_addr_ret,
+(pgtable_iterator_next_fn)(struct drgn_program *prog,
+			   struct pgtable_iterator *it, uint64_t *virt_addr_ret,
 			   uint64_t *phys_addr_ret);
 
 struct drgn_architecture_info {
 	const char *name;
 	enum drgn_architecture arch;
 	enum drgn_platform_flags default_flags;
+	/* API-visible registers. */
 	const struct drgn_register *registers;
+	/* Number of API-visible registers. */
 	size_t num_registers;
+	/*
+	 * Return the API-visible register with the given name, or @c NULL if it
+	 * is not recognized.
+	 */
 	const struct drgn_register *(*register_by_name)(const char *name);
+	/* Internal register layouts indexed by internal register number. */
 	const struct drgn_register_layout *register_layout;
+	/*
+	 * Return the internal register number for the given DWARF register
+	 * number, or @ref DRGN_REGISTER_NUMBER_UNKNOWN if it is not recognized.
+	 */
 	drgn_register_number (*dwarf_regno_to_internal)(uint64_t);
 	/* CFI row containing default rules for DWARF CFI. */
 	const struct drgn_cfi_row *default_dwarf_cfi_row;
@@ -134,6 +146,9 @@ struct drgn_architecture_info {
 	struct drgn_error *(*fallback_unwind)(struct drgn_program *,
 					      struct drgn_register_state *,
 					      struct drgn_register_state **);
+	void (*demangle_return_address)(struct drgn_program *,
+					struct drgn_register_state *,
+					drgn_register_number);
 	/* Given pt_regs as a value buffer object. */
 	struct drgn_error *(*pt_regs_get_initial_registers)(const struct drgn_object *,
 							    struct drgn_register_state **);
@@ -144,15 +159,17 @@ struct drgn_architecture_info {
 	struct drgn_error *(*linux_kernel_get_initial_registers)(const struct drgn_object *,
 								 struct drgn_register_state **);
 	apply_elf_reloc_fn *apply_elf_reloc;
-	struct drgn_error *(*linux_kernel_get_page_offset)(struct drgn_object *);
-	struct drgn_error *(*linux_kernel_get_vmemmap)(struct drgn_object *);
 	struct drgn_error *(*linux_kernel_live_direct_mapping_fallback)(struct drgn_program *,
 									uint64_t *,
 									uint64_t *);
-	/* Size to allocate for pgtable_iterator::arch. */
-	size_t pgtable_iterator_arch_size;
-	/* Initialize pgtable_iterator::arch. */
-	void (*pgtable_iterator_arch_init)(void *buf);
+	/* Allocate a Linux kernel page table iterator. */
+	struct drgn_error *(*linux_kernel_pgtable_iterator_create)(struct drgn_program *,
+								   struct pgtable_iterator **);
+	/* Destroy a Linux kernel page table iterator. */
+	void (*linux_kernel_pgtable_iterator_destroy)(struct pgtable_iterator *);
+	/* (Re)initialize a Linux kernel page table iterator. */
+	void (*linux_kernel_pgtable_iterator_init)(struct drgn_program *,
+						   struct pgtable_iterator *);
 	/* Iterate a (user or kernel) page table in the Linux kernel. */
 	pgtable_iterator_next_fn *linux_kernel_pgtable_iterator_next;
 };
