@@ -44,26 +44,6 @@ struct drgn_symbol;
  * @{
  */
 
-/** The important parts of the VMCOREINFO note of a Linux kernel core. */
-struct vmcoreinfo {
-	/** <tt>uname -r</tt> */
-	char osrelease[128];
-	/** PAGE_SIZE of the kernel. */
-	uint64_t page_size;
-	/**
-	 * The offset from the compiled address of the kernel image to its
-	 * actual address in memory.
-	 *
-	 * This is non-zero if kernel address space layout randomization (KASLR)
-	 * is enabled.
-	 */
-	uint64_t kaslr_offset;
-	/** Kernel page table. */
-	uint64_t swapper_pg_dir;
-	/** Whether 5-level paging was enabled. */
-	bool pgtable_l5_enabled;
-};
-
 struct drgn_thread {
 	struct drgn_program *prog;
 	uint32_t tid;
@@ -154,24 +134,58 @@ struct drgn_program {
 	};
 	struct drgn_thread *main_thread;
 	struct drgn_thread *crashed_thread;
+	/*
+	 * AArch64 instruction pointer authentication code mask, parsed either
+	 * from NT_ARM_PAC_MASK or VMCOREINFO.
+	 */
+	uint64_t aarch64_insn_pac_mask;
 	bool core_dump_notes_cached;
 	bool prefer_orc_unwinder;
 
 	/*
 	 * Linux kernel-specific.
 	 */
-	struct vmcoreinfo vmcoreinfo;
-	/* Cached PAGE_OFFSET. */
-	struct drgn_object page_offset;
+	/* The important parts of the VMCOREINFO note of a Linux kernel core. */
+	struct {
+		/** <tt>uname -r</tt> */
+		char osrelease[128];
+		/** PAGE_SIZE of the kernel. */
+		uint64_t page_size;
+		/**
+		 * The offset from the compiled address of the kernel image to its
+		 * actual address in memory.
+		 *
+		 * This is non-zero if kernel address space layout randomization (KASLR)
+		 * is enabled.
+		 */
+		uint64_t kaslr_offset;
+		/** Kernel page table. */
+		uint64_t swapper_pg_dir;
+		/** Length of mem_section array (i.e., NR_SECTION_ROOTS). */
+		uint64_t mem_section_length;
+		/** VA_BITS on AArch64. */
+		uint64_t va_bits;
+		/** Whether 5-level paging was enabled on x86-64. */
+		bool pgtable_l5_enabled;
+		/** PAGE_SHIFT of the kernel (derived from PAGE_SIZE). */
+		int page_shift;
+	} vmcoreinfo;
+	/*
+	 * Difference between a virtual address in the direct mapping and the
+	 * physical address it maps to.
+	 */
+	uint64_t direct_mapping_offset;
 	/* Cached vmemmap. */
 	struct drgn_object vmemmap;
-	/* Page table iterator for linux_helper_read_vm(). */
+	/* Page table iterator. */
 	struct pgtable_iterator *pgtable_it;
 	/*
-	 * Whether @ref drgn_program::pgtable_it is currently being used. Used
-	 * to prevent address translation from recursing.
+	 * Whether we are currently in address translation. Used to prevent
+	 * address translation from recursing.
 	 */
-	bool pgtable_it_in_use;
+	bool in_address_translation;
+	/* Whether @ref drgn_program::direct_mapping_offset has been cached. */
+	bool direct_mapping_offset_cached;
 };
 
 /** Initialize a @ref drgn_program. */
