@@ -30,7 +30,7 @@ class KernelFlavor(NamedTuple):
     config: str
 
     def localversion(self) -> str:
-        localversion = "-vmtest11"
+        localversion = "-vmtest13"
         # The default flavor should be the "latest" version.
         localversion += ".1" if self.name == "default" else ".0"
         localversion += self.name
@@ -70,6 +70,9 @@ CONFIG_PREEMPT_NONE=y
 ]
 
 
+_PACKAGE_FORMATS = ("tar.zst", "directory")
+
+
 def kconfig(flavor: KernelFlavor) -> str:
     return rf"""# Minimal Linux kernel configuration for booting into vmtest and running drgn
 # tests ({flavor.name} flavor).
@@ -133,6 +136,14 @@ CONFIG_MAGIC_SYSRQ=y
 # For block tests.
 CONFIG_BLK_DEV_LOOP=m
 
+# For BPF tests.
+CONFIG_BPF_SYSCALL=y
+CONFIG_BPF_JIT=y
+CONFIG_BPF_JIT_ALWAYS_ON=y
+CONFIG_CGROUP_BPF=y
+CONFIG_DEBUG_INFO_BTF=y
+CONFIG_DEBUG_INFO_BTF_MODULES=y
+
 # For cgroup tests.
 CONFIG_CGROUPS=y
 
@@ -146,6 +157,9 @@ CONFIG_NAMESPACES=y
 # For nodemask tests.
 CONFIG_NUMA=y
 
+# For slab allocator tests.
+CONFIG_SLAB_FREELIST_HARDENED=y
+
 # For Traffic Control tests.
 CONFIG_NET_SCHED=y
 CONFIG_NET_SCH_PRIO=m
@@ -309,6 +323,7 @@ class KBuild:
             "scripts/Kbuild.include",
             "scripts/Makefile*",
             "scripts/basic/fixdep",
+            "scripts/check-local-export",
             "scripts/gcc-goto.sh",
             "scripts/gcc-version.sh",
             "scripts/mod/modpost",
@@ -348,7 +363,11 @@ class KBuild:
                     dst_path = modules_build_dir / src_path.relative_to(src_dir)
                     dst_path.parent.mkdir(parents=True, exist_ok=True)
                     shutil.copytree(
-                        src_path, dst_path, symlinks=True, dirs_exist_ok=True
+                        src_path,
+                        dst_path,
+                        ignore=shutil.ignore_patterns("*.cmd"),
+                        symlinks=True,
+                        dirs_exist_ok=True,
                     )
 
     async def _test_external_module_build(self, modules_dir: Path) -> None:
@@ -426,17 +445,21 @@ MODULE_LICENSE("GPL");
             if error:
                 raise Exception(f"Command {cmd} output contained error")
 
-    async def package(self, output_dir: Path) -> Path:
+    async def package(self, format: str, output_dir: Path) -> Path:
+        if format not in _PACKAGE_FORMATS:
+            raise ValueError("unknown package format")
+
         make_args = await self._prepare_make()
         kernel_release = await self._kernel_release()
 
-        tarball = output_dir / f"kernel-{kernel_release}.{self._arch}.tar.zst"
+        extension = "" if format == "directory" else ("." + format)
+        package = output_dir / f"kernel-{kernel_release}.{self._arch}{extension}"
 
         logger.info(
             "packaging kernel %s from %s to %s",
             kernel_release,
             self._build_dir,
-            tarball,
+            package,
         )
 
         image_name = (
@@ -479,27 +502,35 @@ MODULE_LICENSE("GPL");
             self._copy_module_build(modules_dir)
             await self._test_external_module_build(modules_dir)
 
-            logger.info("creating tarball")
-            tarball.parent.mkdir(parents=True, exist_ok=True)
-            tar_cmd = ("tar", "-C", str(modules_dir), "-c", ".")
-            zstd_cmd = ("zstd", "-T0", "-19", "-q", "-", "-o", str(tarball), "-f")
-            with pipe_context() as (pipe_r, pipe_w):
-                tar_proc, zstd_proc = await asyncio.gather(
-                    asyncio.create_subprocess_exec(*tar_cmd, stdout=pipe_w),
-                    asyncio.create_subprocess_exec(*zstd_cmd, stdin=pipe_r),
+            package.parent.mkdir(parents=True, exist_ok=True)
+            if format == "directory":
+                logger.info("renaming directory")
+                try:
+                    shutil.rmtree(package)
+                except FileNotFoundError:
+                    pass
+                modules_dir.rename(package)
+            else:
+                logger.info("creating tarball")
+                tar_cmd = ("tar", "-C", str(modules_dir), "-c", ".")
+                zstd_cmd = ("zstd", "-T0", "-19", "-q", "-", "-o", str(package), "-f")
+                with pipe_context() as (pipe_r, pipe_w):
+                    tar_proc, zstd_proc = await asyncio.gather(
+                        asyncio.create_subprocess_exec(*tar_cmd, stdout=pipe_w),
+                        asyncio.create_subprocess_exec(*zstd_cmd, stdin=pipe_r),
+                    )
+                tar_returncode, zstd_returncode = await asyncio.gather(
+                    tar_proc.wait(), zstd_proc.wait()
                 )
-            tar_returncode, zstd_returncode = await asyncio.gather(
-                tar_proc.wait(), zstd_proc.wait()
-            )
-            if tar_returncode != 0:
-                raise CalledProcessError(tar_returncode, tar_cmd)
-            if zstd_returncode != 0:
-                raise CalledProcessError(zstd_returncode, zstd_cmd)
+                if tar_returncode != 0:
+                    raise CalledProcessError(tar_returncode, tar_cmd)
+                if zstd_returncode != 0:
+                    raise CalledProcessError(zstd_returncode, zstd_cmd)
 
         logger.info(
-            "packaged kernel %s from %s to %s", kernel_release, self._build_dir, tarball
+            "packaged kernel %s from %s to %s", kernel_release, self._build_dir, package
         )
-        return tarball
+        return package
 
 
 async def main() -> None:
@@ -535,6 +566,12 @@ async def main() -> None:
         help="also package the built kernel and place it in DIR",
         default=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        "--package-format",
+        choices=_PACKAGE_FORMATS,
+        help='package archive format, or "directory" for an unarchived directory',
+        default=_PACKAGE_FORMATS[0],
+    )
     parser.add_argument(
         "-f",
         "--flavor",
@@ -561,7 +598,7 @@ async def main() -> None:
     kbuild = KBuild(args.kernel_directory, args.build_directory, flavor, "x86_64")
     await kbuild.build()
     if hasattr(args, "package"):
-        await kbuild.package(args.package)
+        await kbuild.package(args.package_format, args.package)
 
 
 if __name__ == "__main__":
